//Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.jdk.core.winHandlers;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

import javax.swing.BorderFactory;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

import net.ajaest.jdk.core.exporters.Exporter;
import net.ajaest.jdk.core.main.ExceptionHandler;
import net.ajaest.jdk.core.main.JDKGUIEngine;
import net.ajaest.jdk.core.main.Launcher;
import net.ajaest.jdk.core.main.Messages;
import net.ajaest.jdk.core.main.ExceptionHandler.KNOWN_EXCEPTIONS;
import net.ajaest.jdk.data.auxi.AllowedStrokeLineEnum;
import net.ajaest.jdk.data.auxi.KanjiComparator;
import net.ajaest.jdk.data.auxi.KanjiEnums;
import net.ajaest.jdk.data.auxi.KanjiStroke;
import net.ajaest.jdk.data.auxi.KanjiEnums.KanjiDicsEnum;
import net.ajaest.jdk.data.auxi.KanjiEnums.KanjiFieldEnum;
import net.ajaest.jdk.data.auxi.KanjiEnums.KanjiReadingTypesEnum;
import net.ajaest.jdk.data.dict.query.KanjiExpression;
import net.ajaest.jdk.data.dict.query.KanjiQuery;
import net.ajaest.jdk.data.dict.sort.KanjiOrdering;
import net.ajaest.jdk.data.dict.sort.KanjiSortExpression;
import net.ajaest.jdk.data.dict.sort.NumberOrderDef;
import net.ajaest.jdk.data.dict.sort.StringOrderDef;
import net.ajaest.jdk.data.kanji.Kanji;
import net.ajaest.jdk.data.kanji.KanjiGraph;
import net.ajaest.jdk.data.kanji.KanjiTag;
import net.ajaest.jdk.gui.auxi.ExportJPanel;
import net.ajaest.jdk.gui.auxi.JPanelFactory;
import net.ajaest.jdk.gui.dialogs.ExportSorterAdderDialog;
import net.ajaest.jdk.gui.dialogs.FileExplorerDialog;
import net.ajaest.jdk.gui.windows.MainWindow;
import net.ajaest.lib.data.Pair;
import net.ajaest.lib.swing.util.FixedTableModel;
import net.ajaest.lib.swing.util.JLineDraw;
import net.ajaest.lib.utils.RowColorCellRenderer;
import net.ajaest.lib.utils.SystemEnums.ISO639ー1;

//TODO: JavaDoc
public class MainWH implements WinHandler {

	// GUI engine
	private JDKGUIEngine jdkGui;
	private MainWindow mw = null;

	// Flags and state
	private Boolean showingKanjiInfo = false;
	private Boolean modifingQuery = false;
	private Boolean sortersModified = true;

	private Integer selectedTab = null;
	private static final Integer TAB_DRAW = 0;
	private static final Integer TAB_CRITERIA = 1;
	@SuppressWarnings("unused")
	private static final Integer TAB_EXPORT = 2;
	@SuppressWarnings("unused")
	private static final Integer TAB_OPTIONS = 3;

	// Win panels
	private JPanel resultPanel = null;
	private JPanel exportResultPanel = null;
	private ExportJPanel exportGridExportJPanel = null;

	// Win properties
	private static final Color TABLE_PRIMARY_BACKGROUNG_COLOR = Color.WHITE;
	private static final Color TABLE_SECONDARY_BACKGROUNG_COLOR = Color.LIGHT_GRAY;
	private static final Dimension fieldInfoDialogDimension = new Dimension(700, 400);

	private FixedTableModel queryTableModel = null;
	private RowColorCellRenderer queryTableCellRenderer = null;

	private FixedTableModel exportSortingTableModel = null;

	private List<KanjiTag> exportSortedList;
	private List<KanjiFieldEnum> exportInvolvedFields;
	private List<Object> exportInvolvedValues;

	public MainWH(JDKGUIEngine jdgGui) {
		this.jdkGui = jdgGui;

		// pre window asignation init: init query tablemodel
		Object[] columnNames = new Object[4];
		columnNames[0] = getMessage(Messages.WINMAIN_QUERY_DATA_columnAndOr);
		columnNames[1] = getMessage(Messages.WINMAIN_QUERY_DATA_columnField);
		columnNames[2] = getMessage(Messages.WINMAIN_QUERY_DATA_columnThatFulfilsThat);
		columnNames[3] = getMessage(Messages.WINMAIN_QUERY_DATA_columnValue);
		int rowCount = 0;
		queryTableModel = new FixedTableModel(columnNames, rowCount);
		String orString = getMessage(Messages.WINMAIN_QUERY_DATA_cellOrString);
		queryTableCellRenderer = new RowColorCellRenderer(queryTableModel, TABLE_PRIMARY_BACKGROUNG_COLOR, TABLE_SECONDARY_BACKGROUNG_COLOR, 0, orString);
	}

	public void addAndQueryButtonPressed() {
    	
		String field = null;
		String fulfils = null;
		String value = null;
		String andOrString = null;

		field = getSelectedMessageFromFieldComboBox();
		fulfils = getSelectedMessageFromFulfilsComboBox();
		value = getSelectedMessageFromValueComboBox();

		if (value != null && !value.equals("") && checkField(field, value)) {
			if (!modifingQuery) {

				Integer rowCount = queryTableModel.getRowCount();

				if (rowCount == 0) {
					mw._getCriteriaTabQueryMaker_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
					mw._getCriteriaTabQueryMaker_OrButton().setEnabled(true);
					mw._getDrawTabAndOr_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
					mw._getDrawTabAndOr_OrButton().setEnabled(true);

					andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellFirstString);
				} else {
					andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellAndString);
				}

				queryTableModel.addRow(new Object[]{andOrString, field, fulfils, value});

			} else if (modifingQuery) {

				Integer selectedRow = null;
				JTable currentTable = null;

				if (selectedTab == TAB_DRAW)
					currentTable = mw._getDrawTab_QueryTable();
				else if (selectedTab == TAB_CRITERIA)
					currentTable = mw._getCriteriaTabQuery_QueryTable();

				selectedRow = currentTable.getSelectedRow();

				if (selectedRow == 0) {

					andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellFirstString);

					if (selectedTab == TAB_DRAW) {
						mw._getDrawTabAndOr_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
						mw._getDrawTabAndOr_OrButton().setEnabled(true);
					} else if (selectedTab == TAB_CRITERIA) {
						mw._getCriteriaTabQueryMaker_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
						mw._getCriteriaTabQueryMaker_OrButton().setEnabled(true);
					}
				} else
					andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellAndString);;

				queryTableModel.setValueAt(andOrString, selectedRow, 0);
				queryTableModel.setValueAt(field, selectedRow, 1);
				queryTableModel.setValueAt(fulfils, selectedRow, 2);
				queryTableModel.setValueAt(value, selectedRow, 3);

				modifingQuery = false;
				setDisableForModifing(true);
			}

			mw._getCriteriaTabQuery_QueryTable().repaint();
			mw._getDrawTab_QueryTable().repaint();
		}
    }

	public void changeExportPath() {
                // Maybe this functionality must be handled by exports dialogs engine
		int selectedExporter = mw._getExportTabAs_FormatComboBox().getSelectedIndex();
		Exporter<KanjiTag> exporter = jdkGui.getJavaDikt().getExporters().get(selectedExporter);

		FileFilter exporterFilters = exporter.getFileChooserFilters();

		FileExplorerDialog fed = new FileExplorerDialog(exporterFilters, mw, true);
		fed.setLocale(new Locale(jdkGui.getOptions().getLanguage()));

		int retValue = fed.showOpenDialog();

		if (retValue == JFileChooser.APPROVE_OPTION) {
			File exportTo = fed.getFileChooser().getSelectedFile();
			mw._getExportTabAs_PathTextField().setText(exportTo.getPath());
			mw._getExportTabPanel().updateUI();
		}

	}

	public void drawAndButtonPressed() {

		if (!mw._getDrawTabDrawer_KanjiDrawPanel().isMouseClicked()) {

			String field = null;
			String fulfils = null;
			String value = null;
			String andOrString = null;

			KanjiGraph kg;
			List<KanjiStroke> ks = new ArrayList<KanjiStroke>();
			JLineDraw draw = mw._getDrawTabDrawer_KanjiDrawPanel().getJLineDraw();

			if (draw != null) {
				List<AllowedStrokeLineEnum> drawnAsterisk = mw._getDrawTabDrawer_KanjiDrawPanel().getJLineDraw().getAsteriskModel();

				// TODO: implement stroke clues
				int order = 1;
				List<AllowedStrokeLineEnum> strokeList;
				for (Pair<Integer, Integer> stroke : draw.getStrokes()) {

					strokeList = new ArrayList<AllowedStrokeLineEnum>();

					for (int i = stroke.getFirst(); i <= stroke.getSecond(); i++) {
						strokeList.add(drawnAsterisk.get(i));
					}

					ks.add(new KanjiStroke(strokeList, order, -1));
					order++;
				}

				kg = new KanjiGraph(ks, null, -1);

				field = getMessage(Messages.KANJI_GRAPH);
				// only equals is allowed
				fulfils = getMessage(Messages.CASE_EQUALS);
				value = kg.toString();

				Integer rowCount = queryTableModel.getRowCount();

				if (rowCount == 0) {
					mw._getCriteriaTabQueryMaker_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
					mw._getCriteriaTabQueryMaker_OrButton().setEnabled(true);
					mw._getDrawTabAndOr_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
					mw._getDrawTabAndOr_OrButton().setEnabled(true);

					andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellFirstString);
				} else {
					andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellAndString);
				}

				queryTableModel.addRow(new Object[]{andOrString, field, fulfils, value});

				mw._getCriteriaTabQuery_QueryTable().repaint();
				mw._getDrawTab_QueryTable().repaint();
			}
		}
	}

	public void drawOrButtonPressed() {

		if (!mw._getDrawTabDrawer_KanjiDrawPanel().isMouseClicked()) {

			String field = null;
			String fulfils = null;
			String value = null;
			String andOrString = null;

			KanjiGraph kg;
			List<KanjiStroke> ks = new ArrayList<KanjiStroke>();
			JLineDraw draw = mw._getDrawTabDrawer_KanjiDrawPanel().getJLineDraw();
			if (draw != null) {
				List<AllowedStrokeLineEnum> drawnAsterisk = mw._getDrawTabDrawer_KanjiDrawPanel().getJLineDraw().getAsteriskModel();

				// TODO: implement stroke clues
				int order = 1;
				List<AllowedStrokeLineEnum> strokeList;
				for (Pair<Integer, Integer> stroke : draw.getStrokes()) {

					strokeList = new ArrayList<AllowedStrokeLineEnum>();

					for (int i = stroke.getFirst(); i <= stroke.getSecond(); i++) {
						strokeList.add(drawnAsterisk.get(i));
					}

					ks.add(new KanjiStroke(strokeList, order, -1));
					order++;
				}

				kg = new KanjiGraph(ks, null, -1);

				field = getMessage(Messages.KANJI_GRAPH);
				// only equals is allowed
				fulfils = getMessage(Messages.CASE_EQUALS);
				andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellOrString);
				value = kg.toString();

				queryTableModel.addRow(new Object[]{andOrString, field, fulfils, value});

				mw._getCriteriaTabQuery_QueryTable().repaint();
				mw._getDrawTab_QueryTable().repaint();
			}
		}
	}
	public void drawRemoveLastLineButtonPressed() {

		if (mw._getDrawTabDrawer_KanjiDrawPanel().isMouseClicked())
			mw._getDrawTabDrawer_KanjiDrawPanel().unclickMouse();
		else
			mw._getDrawTabDrawer_KanjiDrawPanel().removeLastLine();
	}

	public void drawRemoveLastStrokeButtonPressed() {

		if (mw._getDrawTabDrawer_KanjiDrawPanel().isMouseClicked())
			mw._getDrawTabDrawer_KanjiDrawPanel().unclickMouse();

		mw._getDrawTabDrawer_KanjiDrawPanel().removeLastStroke();
	}

    public void drawResetButtonPressed() {

		if (mw._getDrawTabDrawer_KanjiDrawPanel().isMouseClicked())
			mw._getDrawTabDrawer_KanjiDrawPanel().unclickMouse();
		mw._getDrawTabDrawer_KanjiDrawPanel().reset();

		mw._getCriteriaTabQuery_QueryTable().repaint();
		mw._getDrawTab_QueryTable().repaint();
	}

	public void executeDrawQuery() {

		if (mw._getDrawTabDrawer_KanjiDrawPanel().isMouseClicked())
			mw._getDrawTabDrawer_KanjiDrawPanel().unclickMouse();

		setSorterListModified(true);

		executeQuery();
    }

	public void executeQuery() {

		if (queryTableModel.getRowCount() > 0) {
			// elaborates key string values. jdkgui translates it to query
			List<String> boolKeys = new ArrayList<String>();
			List<String> fieldKeys = new ArrayList<String>();
			List<String> caseKeys = new ArrayList<String>();
			List<Object> valueKeys = new ArrayList<Object>();

			List<String> tempList = null;
			Integer rowCount = queryTableModel.getRowCount();
			for (int i = 0; i < rowCount; i++) {

				// and/or column messages can have multiple keys
				tempList = Messages.getKeys((String) queryTableModel.getValueAt(i, 0));
				if (tempList.contains(Messages.WINMAIN_QUERY_DATA_cellFirstString) || tempList.contains(Messages.WINMAIN_QUERY_DATA_cellAndString))
					boolKeys.add(Messages.WINMAIN_QUERY_DATA_cellAndString);
				else
					boolKeys.add(Messages.WINMAIN_QUERY_DATA_cellOrString);

				fieldKeys.add(getSelectedKeyFromFieldComboBox((String) queryTableModel.getValueAt(i, 1)));
				caseKeys.add(Messages.getKey((String) queryTableModel.getValueAt(i, 2)));

					// jdkgui decodes the value literal
					valueKeys.add(queryTableModel.getValueAt(i, 3));

			}

			try {
				List<Integer> currentResults = jdkGui.executeRefQuery(boolKeys, fieldKeys, caseKeys, valueKeys);

				createResultPanels(currentResults);

				setResultPanelBordertText(getMessage(Messages.WINMAIN_RESULTS_BorderTextResults) + ": " + currentResults.size());

				setResultPanelToKanjiToInfoGrid();

			} catch (NumberFormatException e) {
				ExceptionHandler.handleException(e, KNOWN_EXCEPTIONS.NUMBER_CAST_EXCEPTION);

			}
		}
    }

	public void setResultPanelBordertText(String text) {

		mw._getResultsPanel().setBorder(BorderFactory.createTitledBorder(text));
	}

	public void createResultPanels(List<Integer> kt) {

		JScrollPane resultPanelScroll = JPanelFactory.createKanjiGrid(this, kt);
		resultPanel = new JPanel();
		resultPanel.setLayout(new BorderLayout());
		resultPanel.add(mw._getResultControlPanel(), BorderLayout.SOUTH);
		resultPanel.add(resultPanelScroll, BorderLayout.CENTER);

		JScrollPane exportScroll = JPanelFactory.createToggleKanjiGrid(this, kt);
		exportGridExportJPanel = (ExportJPanel) exportScroll.getViewport().getComponent(0);
		exportResultPanel = new JPanel();
		exportResultPanel.setLayout(new BorderLayout());
		exportResultPanel.add(mw._getResultExportControlPanel(), BorderLayout.SOUTH);
		exportResultPanel.add(exportScroll, BorderLayout.CENTER);
	}


    public void exportMoreButtonPressed() {
		
		jdkGui.getExporDialogsWH().exportMoreButtonPressed();
	}

	public void exportRearrangeButtonPressed() {

		List<KanjiTag> selected = null;
		
		if (exportGridExportJPanel != null) {
			selected = new ArrayList<KanjiTag>();

			KanjiQuery q = new KanjiQuery();
			KanjiExpression ke = null;
			for (Integer kanjiRef : exportGridExportJPanel.getSelectedKanji()) {
				ke = q.unicode_value().equal(kanjiRef);
				q = ke.OR();
			}

			selected = this.jdkGui.getJavaDikt().executeQuery(ke);
		}

		if (selected != null && selected.size() > 0) {
			if (sortersModified) {

				exportSortedList = new LinkedList<KanjiTag>(selected);

				KanjiSortExpression kse = createExportSortExpression();

				if (kse != null) {
					Collections.sort(exportSortedList, new KanjiComparator(kse));
					exportSortedList = new ArrayList<KanjiTag>(exportSortedList);

					exportInvolvedFields = kse.getInvolvedFieldsInOrdering();
					exportInvolvedValues = kse.getInvolvedValuesInOrdering();
				} else {
					exportInvolvedFields = new ArrayList<KanjiFieldEnum>();
					exportInvolvedValues = new ArrayList<Object>();
				}

				jdkGui.getExporDialogsWH().getExportRearrangerWH().setSortedList(exportSortedList, exportInvolvedFields, exportInvolvedValues);

				sortersModified = false;
			}

			jdkGui.getExporDialogsWH().invokeRearrangeDialog();
		}
    }
	
	private KanjiSortExpression createExportSortExpression(){
		
		KanjiOrdering ko = new KanjiOrdering();
		KanjiSortExpression kse = null;

		String fieldKey;
		String value = null;
		String valueKey;
		String orderByKey;
		for (int i = 0; i < exportSortingTableModel.getRowCount(); i++) {
			// _____
			fieldKey = (String) exportSortingTableModel.getValueAt(i, 2);
			String[] splitValue = fieldKey.split(" - ");
			if (splitValue.length == 2) {
				fieldKey.replace("ORDER_", "KANJI_");
				value = splitValue[1];
			}

			fieldKey = Messages.getKeyInList(splitValue[0], Messages.LIST_ORDENABLE_FIELDS);
			// _____

			orderByKey = (String) exportSortingTableModel.getValueAt(i, 1);
			switch (KanjiEnums.getFieldType(fieldKey.replace("ORDER_", "KANJI_"))) {
				case INTEGER :
					orderByKey = Messages.getKeyInList(orderByKey, Messages.LIST_ORDENABLE_METHODS_INTEGER);
					break;
				case STRING :
					orderByKey = Messages.getKeyInList(orderByKey, Messages.LIST_ORDENABLE_METHODS_STRING);
					break;
			}
			// _____

			if (fieldKey.equals(Messages.ORDER_CLASSIC_NELSON)) {
				kse = selectOrderBy(ko.sort_by_Nelson_radical(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_CLASSIC_RADICAL)) {
				kse = selectOrderBy(ko.sort_by_classic_radical(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_DE_ROO)) {
				kse = selectOrderBy(ko.sort_by_De_Roo_code(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_DIC_INDEX)) {
				valueKey = Messages.getKeyInList(value, Messages.LIST_CRITERIA_VALUES_DIC);
				KanjiDicsEnum kde = KanjiDicsEnum.valueOf(valueKey);
				kse = selectOrderBy(ko.sort_by_dictionary_reference().dic_enum_equals(kde), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_FOUR_CORNER)) {
				kse = selectOrderBy(ko.sort_by_four_corner_code(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_FREQUENCY)) {
				kse = selectOrderBy(ko.sort_by_frequency(), orderByKey);
			} else if (fieldKey.equals(Messages.ORDER_GRADE)) {
				kse = selectOrderBy(ko.sort_by_grade(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_GRAPH)) {
				kse = selectOrderBy(ko.sort_by_graph(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_JIS_CHARSET)) {
				kse = selectOrderBy(ko.sort_by_JIS_charset(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_JIS_CODE)) {
				kse = selectOrderBy(ko.sort_by_JIS_code(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_MEANING)) {
				ISO639ー1 lang = ISO639ー1.valueOf(jdkGui.getOptions().getLanguage().toUpperCase());
				kse = selectOrderBy(ko.sort_by_first_meaning().languaje_enum_equals(lang), orderByKey);
			} else if (fieldKey.equals(Messages.ORDER_READING)) {
				valueKey = Messages.getKeyInList(value, Messages.LIST_CRITERIA_VALUES_READING_EXTENDED);
				KanjiReadingTypesEnum kre = KanjiReadingTypesEnum.valueOf(valueKey);
				kse = selectOrderBy(ko.sort_by_first_reading().reading_enum_equals(kre), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_SKIP)) {
				kse = selectOrderBy(ko.sort_by_SKIP_code(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_SPAHN_HADAMITZKY)) {
				kse = selectOrderBy(ko.sort_by_SpahnーHadamitzky_code(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_STROKE_COUNT)) {
				kse = selectOrderBy(ko.sort_by_stroke_count(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_UNICODE_VALUE)) {
				kse = selectOrderBy(ko.sort_by_Unicode(), orderByKey);
				ko = kse.and_if_equals();
			}

			// _____
		}
		
		return kse;
	}

	private KanjiSortExpression selectOrderBy(StringOrderDef sod, String orderByKey) {

		KanjiSortExpression kse = null;

		if (orderByKey.equals(Messages.ORDERMETHODSSTR_ALPHABETICALLY)) {
			kse = sod.alphabetically();
		} else if (orderByKey.equals(Messages.ORDERMETHODSSTR_ALPHABETICALLY_INVERSE)) {
			kse = sod.alphabetically_inverse();
		} else if (orderByKey.equals(Messages.ORDERMETHODSINT_RAMDOM)) {
			kse = sod.ramdomly();
		} else if (orderByKey.equals(Messages.ORDERMETHODSSTR_RAMDOM)) {
			kse = sod.ramdomly();
		}

		return kse;
	}

	private KanjiSortExpression selectOrderBy(NumberOrderDef sod, String orderByKey) {

		KanjiSortExpression kse = null;

		if (orderByKey.equals(Messages.ORDERMETHODSINT_FROM_GREATEST_TO_LEAST)) {
			kse = sod.from_greatest_to_least();
		} else if (orderByKey.equals(Messages.ORDERMETHODSINT_FROM_LEAST_TO_GREATEST)) {
			kse = sod.from_least_to_greatest();
		} else if (orderByKey.equals(Messages.ORDERMETHODSINT_RAMDOM)) {
			kse = sod.ramdomly();
		} else if (orderByKey.equals(Messages.ORDERMETHODSSTR_RAMDOM)) {
			kse = sod.ramdomly();
		}

		return kse;
	}

	public void exportResultPanelInvertButtonPressed() {

		if (exportGridExportJPanel != null)
			exportGridExportJPanel.invertSelection();
	}

    public void exportResultPanelResetButtonPressed() {

		if (exportGridExportJPanel != null)
			exportGridExportJPanel.setAll(false);
	}

	public void exportResultPanelSelectAllButtonPressed() {

		if (exportGridExportJPanel != null)
			exportGridExportJPanel.setAll(true);
	}

	public ComboBoxModel getCriteriaFieldComboBoxModel() {

		String[] comboMessages;
		if (jdkGui.getOptions().getExtendedInfo()) {
			comboMessages = Messages.getList(Messages.LIST_CRITERIA_FIELD_EXTENDED);
		} else
			comboMessages = Messages.getList(Messages.LIST_CRITERIA_FIELD_BASIC);
		
		return new DefaultComboBoxModel(comboMessages);
	}

	public ComboBoxModel getExportFormatModel() {

		String[] exporterNames = new String[jdkGui.getJavaDikt().getExporters().size()];

		for (int i = 0; i < jdkGui.getJavaDikt().getExporters().size(); i++) {

			exporterNames[i] = jdkGui.getJavaDikt().getExporters().get(i).getName();
		}

		ComboBoxModel cbm = new DefaultComboBoxModel(exporterNames);
		if (jdkGui.getJavaDikt().getExporters().size() > 0)
			cbm.setSelectedItem(jdkGui.getJavaDikt().getExporters().get(0).getName());

		return cbm;
	}

    public TableModel getExportSorterTableModel() {

		Object[] columnNames = new Object[3];
		columnNames[0] = getMessage(Messages.WINMAIN_EXPORT_SORTING_columnLevel);
		columnNames[1] = getMessage(Messages.WINMAIN_EXPORT_SORTING_columnOrderBy);
		columnNames[2] = getMessage(Messages.WINMAIN_EXPORT_SORTING_columnField);

		int rowCount = 0;

		exportSortingTableModel = new FixedTableModel(columnNames, rowCount);

		return exportSortingTableModel;
	}

	public ComboBoxModel getExportStyleModel() {

		String[] stylerNames = new String[jdkGui.getJavaDikt().getExporters().size()];

		ComboBoxModel cbm;
		
		if (jdkGui.getJavaDikt().getExporters().size() > 0) {

			Exporter<?> temp = jdkGui.getJavaDikt().getExporters().get(0);
			for (int i = 0; i < temp.getStylers().size(); i++) {

				stylerNames[i] = temp.getStylers().get(i).getName();
			}

			cbm = new DefaultComboBoxModel(stylerNames);
			if (temp.getStylers().size() > 0)
				cbm.setSelectedItem(temp.getStylers().get(0).getName());
		} else {
			cbm = new DefaultComboBoxModel(stylerNames);
		}

		return cbm;
	}

	public List<? extends Image> getIcons() {

		return jdkGui.getIcons();
	}

	public MainWindow getWindow() {
		return mw;
	}

    public String getMessage(String message) {

		return Messages.get(message);
	}

	public ComboBoxModel getOptionLanguageComboBoxModel() {

		return new DefaultComboBoxModel(Messages.getList(Messages.LIST_LANGUAGES_SUPPORTED));
	}

	public TableModel getQueryTableModel() {

		return queryTableModel;
	}

	public TableCellRenderer getQueryTableCellRenderer() {

		return queryTableCellRenderer;
	}

    private String getSelectedKeyFromFieldComboBox(String selectedMessage) {

		List<String> keys = Messages.getKeys(selectedMessage);
		String selectedKey = "";
		

		for (String key : keys) {
			if (key.contains("KANJI_")) {
				selectedKey = key;
				break;
			}
		}

		return selectedKey;
	}
    
	private String getSelectedMessageFromFieldComboBox() {

		int selectedItemIndex = mw._getCriteriaTabQueryMaker_FieldComboBox().getSelectedIndex();

		String[] currentBoxList;
		if (jdkGui.getOptions().getExtendedInfo())
			currentBoxList = Messages.getList(Messages.LIST_CRITERIA_FIELD_EXTENDED);
		else
			currentBoxList = Messages.getList(Messages.LIST_CRITERIA_FIELD_BASIC);

		return currentBoxList[selectedItemIndex];
	}

	private String getSelectedMessageFromFulfilsComboBox() {

		String fieldSelectedMessage = getSelectedMessageFromFieldComboBox();
		String fieldKey = getSelectedKeyFromFieldComboBox(fieldSelectedMessage);
		
		String[] currentBoxList = null;
		switch (KanjiEnums.getFieldType(fieldKey)) {
			case INTEGER :
			case ORDENABLE_STRING :
				currentBoxList = Messages.getList(Messages.LIST_CRITERIA_CASES_INT);
				break;
			case STRING :
				currentBoxList = Messages.getList(Messages.LIST_CRITERIA_CASES_STRING);
				break;
		}

		return currentBoxList[mw._getCriteriaTabQueryMaker_FulfilsComboBox().getSelectedIndex()];
	}

	private String getSelectedMessageFromValueComboBox() {

		String valueSelectedMessage = null;

		if (mw._getCriteriaTabQueryMaker_ValueComboBox().isEditable()) {

			valueSelectedMessage = (String) mw._getCriteriaTabQueryMaker_ValueComboBox().getSelectedItem();
		} else {


			int selectedValueIndex = mw._getCriteriaTabQueryMaker_ValueComboBox().getSelectedIndex();

			String selectedKey;

			if (jdkGui.getOptions().getExtendedInfo()) {
				selectedKey = Messages.getKeyInList(getSelectedMessageFromFieldComboBox(), Messages.LIST_CRITERIA_FIELD_EXTENDED);
			} else {
				selectedKey = Messages.getKeyInList(getSelectedMessageFromFieldComboBox(), Messages.LIST_CRITERIA_FIELD_BASIC);
			}

			if (selectedKey.equals(Messages.KANJI_DIC_NAME))
				valueSelectedMessage = Messages.getList(Messages.LIST_CRITERIA_VALUES_DIC)[selectedValueIndex];
			else if (selectedKey.startsWith("KANJI_CLASSIC_RADICAL"))
				valueSelectedMessage = Messages.getList(Messages.LIST_CRITERIA_VALUES_RADICALS)[selectedValueIndex];
			else if (selectedKey.equals(Messages.KANJI_JIS_CHARSET))
				valueSelectedMessage = Messages.getList(Messages.LIST_CRITERIA_VALUES_JIS)[selectedValueIndex];
			else if (selectedKey.equals(Messages.KANJI_READING_TYPE)) {
				if (jdkGui.getOptions().getExtendedInfo())
					valueSelectedMessage = Messages.getList(Messages.LIST_CRITERIA_VALUES_READING_EXTENDED)[selectedValueIndex];
				else
					valueSelectedMessage = Messages.getList(Messages.LIST_CRITERIA_VALUES_READING_BASIC)[selectedValueIndex];
			}
		}

		return valueSelectedMessage;
	}

	private void initialize() {
		selectedTab = 0;
		// pos window asignation init: query comboboxes
		mw._getCriteriaTabQueryMaker_FieldComboBox().setSelectedIndex(0);
		updateChoosableQueryComboBoxes();

	}

	public void invokeKanjiInfoWindow(Integer kanjiRef) {
		jdkGui.getKanjiInfoWinEngine().invokeWindow(kanjiRef);
	}

	public void kanjiFileChoose() {

		FileFilter kanjiDictFileFilter = new FileFilter() {
			@Override
			public boolean accept(File f) {

				return f.isDirectory() || f.isFile() && f.getName().endsWith(".jdk");
			}
			@Override
			public String getDescription() {
				return Messages.get(Messages.FILE_DESCRIPTION_JDK_KANJI_FILE);
			}
		};
		
		FileExplorerDialog fed = new FileExplorerDialog(kanjiDictFileFilter, mw, true);
		fed.setLocale(new Locale(jdkGui.getOptions().getLanguage()));

		int retValue = fed.showOpenDialog();

		if (retValue == JFileChooser.APPROVE_OPTION) {
			File kanjiDict = fed.getFileChooser().getSelectedFile();
			mw._getOptionsTab_KanjiDictPathTextField().setText(kanjiDict.getAbsolutePath());
			mw._getOptionsTabPanel().updateUI();
		}
    }

    public void orQueryButtonPressed() {

		String field = null;
		String fulfils = null;
		String value = null;
		String andOrString = null;

		andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellOrString);
		field = getSelectedMessageFromFieldComboBox();
		fulfils = getSelectedMessageFromFulfilsComboBox();
		value = getSelectedMessageFromValueComboBox();

		if (value != null && !value.equals("") && checkField(field, value)) {
			if (!modifingQuery) {
				queryTableModel.addRow(new Object[]{andOrString, field, fulfils, value});
			} else if (modifingQuery) {

				Integer selectedRow = null;
				JTable currentTable = null;

				if (selectedTab == TAB_DRAW)
					currentTable = mw._getDrawTab_QueryTable();
				else if (selectedTab == TAB_CRITERIA)
					currentTable = mw._getCriteriaTabQuery_QueryTable();

				selectedRow = currentTable.getSelectedRow();

				queryTableModel.setValueAt(andOrString, selectedRow, 0);
				queryTableModel.setValueAt(field, selectedRow, 1);
				queryTableModel.setValueAt(fulfils, selectedRow, 2);
				queryTableModel.setValueAt(value, selectedRow, 3);

				modifingQuery = false;
				setDisableForModifing(true);
			}

			mw._getCriteriaTabQuery_QueryTable().repaint();
			mw._getDrawTab_QueryTable().repaint();
		}
    }

	public void QueryModifierModifyButtonPressed() {

		Integer selectedRow = null;
		JTable currentTable = null;

		if (selectedTab == TAB_DRAW)
			currentTable = mw._getDrawTab_QueryTable();
		else if (selectedTab == TAB_CRITERIA)
			currentTable = mw._getCriteriaTabQuery_QueryTable();


		selectedRow = currentTable.getSelectedRow();

		// Changes the selection in order to mark only the row that is being
		// modified
		currentTable.getSelectionModel().setSelectionInterval(selectedRow, selectedRow);
		currentTable.updateUI();
		// TODO: you can do much better!
		if (selectedRow != -1) {

			if (selectedRow == 0) {
				if (selectedTab == TAB_DRAW) {
					// No "and" or "or" for first column is allowed
					mw._getDrawTabAndOr_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerChangeText));
					mw._getDrawTabAndOr_OrButton().setEnabled(false);

				} else if (selectedTab == TAB_CRITERIA) {
					// No "and" or "or" for first column is allowed
					mw._getCriteriaTabQueryMaker_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerChangeText));
					mw._getCriteriaTabQueryMaker_OrButton().setEnabled(false);
				}
			}

			if (modifingQuery) {
				setDisableForModifing(true);
				// restores and/or buttons
				if (selectedRow == 0) {
					if (selectedTab == TAB_DRAW) {
						// No "and" or "or" for first column is allowed
						mw._getDrawTabAndOr_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
						mw._getDrawTabAndOr_OrButton().setEnabled(true);

					} else if (selectedTab == TAB_CRITERIA) {
						// No "and" or "or" for first column is allowed
						mw._getCriteriaTabQueryMaker_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
						mw._getCriteriaTabQueryMaker_OrButton().setEnabled(true);
					}
				}
			} else 
				setDisableForModifing(false);
			
			modifingQuery = !modifingQuery;
		}

		mw._getCriteriaTabQuery_QueryTable().repaint();
		mw._getDrawTab_QueryTable().repaint();
	}

	public void QueryModifierRemoveButtonPressed() {
		
		int[] selectedRow = null;
		JTable currentTable = null;

		if (selectedTab == TAB_DRAW)
			currentTable = mw._getDrawTab_QueryTable();
		else if (selectedTab == TAB_CRITERIA)
			currentTable = mw._getCriteriaTabQuery_QueryTable();

		selectedRow = currentTable.getSelectedRows();

		int lastRemovedRow = 0;
		
		for(int i=0; i<selectedRow.length; i++){
			// -i because when a row is deleted the next rows change their
			// position
			queryTableModel.removeRow(selectedRow[i] - i);
			// We cannot expected that the rows are sorted
			if (selectedRow[i] > lastRemovedRow)
				lastRemovedRow = selectedRow[i]-i;
		}
		
		if (queryTableModel.getRowCount() > lastRemovedRow)
			currentTable.changeSelection(lastRemovedRow, 1, false, false);
		else
			currentTable.changeSelection(currentTable.getRowCount() - 1, 1, false, false);

		if (queryTableModel.getRowCount() == 0) {
			mw._getCriteriaTabQueryMaker_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAddText));
			mw._getCriteriaTabQueryMaker_OrButton().setEnabled(false);
			mw._getDrawTabAndOr_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAddText));
			mw._getDrawTabAndOr_OrButton().setEnabled(false);
		} else
			// if the first row is deleted, the new first row and/or attribute
			// must be "-"
			queryTableModel.setValueAt(getMessage(Messages.WINMAIN_QUERY_DATA_cellFirstString), 0, 0);

		mw._getCriteriaTabQuery_QueryTable().repaint();
		mw._getDrawTab_QueryTable().repaint();
	}

	public void QueryModifierResetButtonPressed() {

		int size = queryTableModel.getRowCount();

		for (int i = 0; i < size; i++) {
			queryTableModel.removeRow(queryTableModel.getRowCount() - 1);

		}

		mw._getCriteriaTabQueryMaker_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAddText));
		mw._getCriteriaTabQueryMaker_OrButton().setEnabled(false);
		mw._getDrawTabAndOr_AndAddButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAddText));
		mw._getDrawTabAndOr_OrButton().setEnabled(false);
	}

	public void saveConfig() {

		// TODO: not restart needed to apply changes

		String lang = Messages.getKey((String) mw._getOptionsTab_LanguajeComboBox().getSelectedItem());
		lang = lang.substring("LANGUAJE_".length());
    	String kanjipath = mw._getOptionsTab_KanjiDictPathTextField().getText();
		String strokepath = mw._getOptionsTab_StrokesPathTextField().getText();
    	Boolean extInfo = mw._getOptionsTab_ExtendedInfoCheckBox().isSelected();
    	Boolean extKanji = mw._getOptionsTab_ExtendedIKanjiListCheckBox().isSelected();
    	Boolean romaji = mw._getOptionsTab_KanaToRomajiCheckBox().isSelected();

		jdkGui.getJavaDikt().saveOptions(lang, kanjipath, strokepath, extInfo, extKanji, romaji);

		jdkGui.reinitEngine();
    }

	public void setDisableForModifing(Boolean disable) {
		
		mw._getControlsTabbedPane().setEnabled(disable);

		if (selectedTab == TAB_CRITERIA) {
			mw._getCriteriaQueryControlModify_RemoveButton().setEnabled(disable);
			mw._getCriteriaQueryControlModify_ResetButton().setEnabled(disable);
			mw._getCriteriaTabQueryControl_ExecuteButton().setEnabled(disable);
			mw._getCriteriaTabQuery_QueryTable().setEnabled(disable);
			mw._getCriteriaTabQueryMaker_InfoButton().setEnabled(disable);
			// I don't know why renderer dissapears when disabled
			mw._getCriteriaTabQuery_QueryTable().setDefaultRenderer(Object.class, getQueryTableCellRenderer());

		} else if (selectedTab == TAB_DRAW) {
			mw._getDrawTabQueryControl_RemoveButton().setEnabled(disable);
			mw._getDrawTabQueryControl_ResetButton().setEnabled(disable);
			mw._getDrawTab_ExecuteButton().setEnabled(disable);
			mw._getDrawTab_QueryTable().setEnabled(disable);
			mw._getCriteriaTabQueryMaker_InfoButton().setEnabled(disable);
			// I don't know why renderer dissapears when disabled
			mw._getDrawTab_QueryTable().setDefaultRenderer(Object.class, getQueryTableCellRenderer());
		}

		mw._getCriteriaTabQuery_QueryTable().repaint();
		mw._getDrawTab_QueryTable().repaint();
	}

	private void setResultPanelToKanjiExportGrid() {

		JPanel results = mw._getResultsPanel();

		// remove void panel if first or previous panel
		results.remove(0);

		showingKanjiInfo = false;

		results.add(exportResultPanel, BorderLayout.CENTER);

		results.updateUI();
	}

	private void setResultPanelToKanjiToInfoGrid() {

		JPanel results = mw._getResultsPanel();

		// remove void panel if first or previous panel
		results.remove(0);

		showingKanjiInfo = true;

		results.add(resultPanel, BorderLayout.CENTER);

		results.updateUI();
	}

	public void setWindow(MainWindow mw) {
		this.mw = mw;
		initialize();

	}

	public void strokeFileChoose() {
        FileFilter kanjiDictFileFilter = new FileFilter() {
			@Override
			public boolean accept(File f) {

				return f.isDirectory() || f.isFile() && f.getName().endsWith(".zobj");
			}
			@Override
			public String getDescription() {
				return Messages.get(Messages.FILE_DESCRIPTION_ZOBJ_STROKE_FILE);
			}
		};

		FileExplorerDialog fed = new FileExplorerDialog(kanjiDictFileFilter, mw, true);
		fed.setLocale(new Locale(jdkGui.getOptions().getLanguage()));

		int retValue = fed.showOpenDialog();

		if (retValue == JFileChooser.APPROVE_OPTION) {
			File kanjiDict = fed.getFileChooser().getSelectedFile();
			mw._getOptionsTab_KanjiDictPathTextField().setText(kanjiDict.getAbsolutePath());
			mw._getOptionsTabPanel().updateUI();
		}
    }

    public void tabChangedActionPerformed(int index) {
		
    	selectedTab = index;

		if (exportResultPanel != null) {

			switch (selectedTab) {
				// stroke and criteria tab
				case 0 :
				case 1 :
					// if getBufferedKanjiInfoGrid() exists but another panel is
					// being displayed in results panel
					if (!showingKanjiInfo)
						setResultPanelToKanjiToInfoGrid();

					break;
				// export tab
				case 2 :
					setResultPanelToKanjiExportGrid();
					break;
				// options tab
				case 3 :
					updateOptionPanel();
					break;
			}

		} else {
			switch (selectedTab) {
				case 3 :
					updateOptionPanel();
					break;
			}

		}
    }

	public void updateChoosableQueryComboBoxes() {
    	
		String selectedMessage = getSelectedMessageFromFieldComboBox();
		String selectedKey = getSelectedKeyFromFieldComboBox(selectedMessage);

		mw._getCriteriaTabQueryMaker_InfoButton().setEnabled(Messages.hasMessage("INFO_" + selectedKey));

		ComboBoxModel fullfilModel = null;
		ComboBoxModel valueModel = null;

		switch (KanjiEnums.getFieldType(selectedKey)) {
			case INTEGER:
			case ORDENABLE_STRING :
				fullfilModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_CASES_INT));
				break;
			case STRING:
				fullfilModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_CASES_STRING));
				break;
			case INTEGER_LIST:
				fullfilModel = new DefaultComboBoxModel();
				break;

		}

		if (selectedKey.equals(Messages.KANJI_DIC_NAME)) {

			valueModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_VALUES_DIC));
			mw._getCriteriaTabQueryMaker_ValueComboBox().setEditable(false);

		} else if (selectedKey.endsWith(Messages.KANJI_JIS_CHARSET)) {

			valueModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_VALUES_JIS));
			mw._getCriteriaTabQueryMaker_ValueComboBox().setEditable(false);

		} else if (selectedKey.equals(Messages.KANJI_READING_TYPE)) {

			if (jdkGui.getOptions().getExtendedInfo())
				valueModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_VALUES_READING_EXTENDED));
			else
				valueModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_VALUES_READING_BASIC));

			mw._getCriteriaTabQueryMaker_ValueComboBox().setEditable(false);
		} else if (selectedKey.equals(Messages.KANJI_CLASSIC_RADICAL)) {

			valueModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_VALUES_RADICALS));
			mw._getCriteriaTabQueryMaker_ValueComboBox().setEditable(false);

		} else {
			if (mw._getCriteriaTabQueryMaker_ValueComboBox().isEditable())
				valueModel = mw._getCriteriaTabQueryMaker_ValueComboBox().getModel();
			else
				valueModel = new DefaultComboBoxModel();

			mw._getCriteriaTabQueryMaker_ValueComboBox().setEditable(true);
		}

		mw._getCriteriaTabQueryMaker_FulfilsComboBox().setModel(fullfilModel);
		mw._getCriteriaTabQueryMaker_FulfilsComboBox().setSelectedIndex(0);
		mw._getCriteriaTabQueryMaker_FulfilsComboBox().setEnabled(true);

		mw._getCriteriaTabQueryMaker_ValueComboBox().setModel(valueModel);
		if (!mw._getCriteriaTabQueryMaker_ValueComboBox().isEditable())
			mw._getCriteriaTabQueryMaker_FulfilsComboBox().setSelectedIndex(0);
		mw._getCriteriaTabQueryMaker_ValueComboBox().setEnabled(true);

    }

	public void updateOptionPanel() {
    	
		mw._getOptionsTab_LanguajeComboBox().setSelectedItem(jdkGui.getOptions().getLanguage());
		mw._getOptionsTab_KanjiDictPathTextField().setText(jdkGui.getOptions().getKanjiFile().getAbsolutePath());
		mw._getOptionsTab_StrokesPathTextField().setText(jdkGui.getOptions().getTreeFile().getAbsolutePath());
		mw._getOptionsTab_ExtendedInfoCheckBox().setSelected(jdkGui.getOptions().getExtendedInfo());
		mw._getOptionsTab_ExtendedIKanjiListCheckBox().setSelected(jdkGui.getOptions().getExtendedIKanjiList());
		mw._getOptionsTab_KanaToRomajiCheckBox().setSelected(jdkGui.getOptions().getRomanized());

		mw._getOptionsTabPanel().updateUI();
    }

	public static void main(String... args) {

		Launcher.main();
	}

	public void exportSorterAddButtonPressed() {
		jdkGui.getExporDialogsWH().invokeAddSorterDialog();

	}

	public void exportSorterResetButtonPressed() {
		mw._getExportTabSorting_SortingTable().setModel(getExportSorterTableModel());

	}

	public void exportSorterRemoveButtonPressed() {

		int[] selectedRow = mw._getExportTabSorting_SortingTable().getSelectedRows();

		int lastRemovedRow = 0;

		for (int i = 0; i < selectedRow.length; i++) {
			// -i because when a row is deleted the next rows change their
			// position
			exportSortingTableModel.removeRow(selectedRow[i] - i);
			// We cannot expected that the rows are sorted
			if (selectedRow[i] > lastRemovedRow)
				lastRemovedRow = selectedRow[i] - i;
		}

		if (exportSortingTableModel.getRowCount() > lastRemovedRow)
			mw._getExportTabSorting_SortingTable().changeSelection(lastRemovedRow, 1, false, false);
		else
			mw._getExportTabSorting_SortingTable().changeSelection(exportSortingTableModel.getRowCount() - 1, 1, false, false);

		// level literals must be re-written

		for (int i = 0; i < exportSortingTableModel.getRowCount(); i++) {
			exportSortingTableModel.setValueAt(new Integer(i + 1).toString(), i, 0);
		}

		// if add sorter is visible level combo box must be modified
		
		ExportSorterAdderDialog esaD = jdkGui.getExporDialogsWH().getExportSorterAdderWH().getExportSorterAdderDialog();
		
		if (esaD.isVisible()) {
			esaD.getLevelComboBox().setModel(jdkGui.getExporDialogsWH().getExportSorterAdderWH().getLevelComboBoxModel());
		}

		esaD.getLevelComboBox().setSelectedIndex(exportSortingTableModel.getRowCount());

		mw._getExportTabSorting_SortingTable().repaint();
	}

	public void setSorterListModified(boolean b) {
		this.exportSortedList = null;
		this.sortersModified = b;
	}

	public void exportExportButtonPressed() {

		String path = mw._getExportTabAs_PathTextField().getText();

		if (path != null && !path.equals("")) {

			Exporter<KanjiTag> exporter;

			int exporterIdx = mw._getExportTabAs_FormatComboBox().getSelectedIndex();
			exporter = jdkGui.getJavaDikt().getExporters().get(exporterIdx);
			int stylerIdx = mw._getExportTabAs_StyleComboBox().getSelectedIndex();
			exporter.setSelectedStyler(stylerIdx);

			File f = new File(path);
			
			exporter.setExportFile(f);

			if (exportSortedList == null) {
				exportSortedList = new ArrayList<KanjiTag>();

				KanjiQuery q = new KanjiQuery();
				KanjiExpression ke = null;
				for (Integer kanjiRef : exportGridExportJPanel.getSelectedKanji()) {
					ke = q.unicode_value().equal(kanjiRef);
					q = ke.OR();
				}

				exportSortedList = this.jdkGui.getJavaDikt().executeQuery(ke);
			}

			exporter.export(exportSortedList);

			if (exporter.getExportException() == null) {
				jdkGui.getInfoDialgosEngine().invokeInfoDialog(getMessage(Messages.INFODIALOG_title_info), getMessage(Messages.INFODIALOG_infoMessage_exportSuccess), null);
			} else {
				ExceptionHandler.handleException(exporter.getExportException(), KNOWN_EXCEPTIONS.EXPORT_IOEXCEPTION);
			}
			exportSortedList = null;
		}

	}

	public void setExportSortedList(List<KanjiTag> sortedList) {

		this.exportSortedList = sortedList;
	}

	private boolean checkField(String message, String value) {

		// TODO: replace using TextComponent validators

		boolean passCheck = true;

		String key = Messages.getKeyInList(message, Messages.LIST_CRITERIA_FIELD_EXTENDED);

		switch (KanjiEnums.getFieldType(key)) {
			case INTEGER :
			case INTEGER_LIST :

				if (key.equals("KANJI_CLASSIC_RADICAL")) {

					passCheck = true;

				} else if (key.equals(Messages.KANJI_LITERAL)) {

					int codePoint = Character.codePointAt(value.toCharArray(), 0);
					passCheck = Kanji.isKanji(codePoint) && value.length() == 1;

					if (!passCheck)
						ExceptionHandler.handleException(new IllegalArgumentException(), KNOWN_EXCEPTIONS.NOT_A_KANJI);
				} else if (key.equals("KANJI_UNICODE_VALUE")) {
					
					try{
						Integer.valueOf(value, 16);
					} catch (NumberFormatException e) {
						passCheck = false;
						ExceptionHandler.handleException(new NumberFormatException(), KNOWN_EXCEPTIONS.IS_NOT_HEX);
					}
					
				} else {

					try{
						new Integer(value);
					} catch (NumberFormatException e) {
						passCheck = false;
						ExceptionHandler.handleException(new NumberFormatException(), KNOWN_EXCEPTIONS.NUMBER_CAST_EXCEPTION);
					}
				}
				break;
			case STRING :
				if (key.equals(Messages.KANJI_GRAPH)) {
					String astGraphStr = new String(value);
					passCheck = Character.isUpperCase(astGraphStr.toCharArray()[0]);
					astGraphStr = astGraphStr.toLowerCase();
					astGraphStr = astGraphStr.replaceAll("[abcdefghi]", "");
					passCheck = passCheck && (astGraphStr.length() == 0);
					if (!passCheck) {
						ExceptionHandler.handleException(new IllegalArgumentException(), KNOWN_EXCEPTIONS.GRAPH_CAST_EXCEPTION);
					}

				} else {
					passCheck = true;
				}
				break;
			default :
				passCheck = true;
				break;
		}

		return passCheck;
	}

	public void optionsAboutButtonPressed() {
		jdkGui.getAboutWH().invokeAboutWindow();
	}

	public Font getUnicodeFont() {

		return jdkGui.getUnicodeFont();
	}

    public void exportResultDisposeAllWindowsButtonPressed() {
        jdkGui.getKanjiInfoWinEngine().disposeAllWindows();
    }

	public JDKGUIEngine getJDKGuiEngine() {

		return jdkGui;
	}

    public void _criteriaTabQueryMaker_InfoButtonPressed() {

		String selectedComboBoxMessage = getSelectedMessageFromFieldComboBox();
		String infoMessageKey = "INFO_" + Messages.getKeyInList(selectedComboBoxMessage, Messages.LIST_CRITERIA_FIELD_EXTENDED);
		String infoMessage = Messages.get(infoMessageKey);
		Messages.localePrintln("**MainWH: retrieving info message ,\"%s\"", infoMessageKey);
		this.jdkGui.getInfoDialgosEngine().invokeBorderedInfoDialog(selectedComboBoxMessage, infoMessage, fieldInfoDialogDimension, Messages.get(Messages.INFODIALOG_infoBorderText), true);
    }
}
