//    Copyright (C] 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option] any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.jdk.data.auxi;

import java.io.Serializable;

/**
 * Enumerator of posible lines for defining strokes.
 * 
 * @author Luis Alfonso Arce González
 * 
 */
public enum AllowedStrokeLineEnum implements Serializable {
	// Lowercase means change of direcction in the same stroke, for example
	// く is "Ac", stroke formed with lines A and C

	/**
	 * 
	 * <br>{@code [ ][ ][b]}<br> {@code [ ][*][ ]}<br> {@code [e][ ][ ]}<br>
	 * <br> {@code b =} begin<br> {@code e =} end<br>
	 * <br>
	 * Diagonal, from [0,2] to [2,0]. Example: ノ = <b>A</b>
	 */
	A,

	/**
	 * 
	 * <br>{@code [b]}<br> {@code [*]}<br> {@code [e]}<br>
	 * <br> {@code b =} begin<br> {@code e =} end<br>
	 * <br>
	 * Vertical, from [0,0] to [2,0]. Example: リ = <b>BB</b>a
	 */
	B,
	/**
	 * 
	 * <br>{@code [b][ ][ ]}<br> {@code [ ][*][ ]}<br> {@code [ ][ ][e]}<br>
	 * <br> {@code b =} begin<br> {@code e =} end<br>
	 * <br>
	 * Diagonal, from [0,0] to [2,2]. Example: く = A<b>c</b>
	 */

	C,
	/**
	 * 
	 * <br>{@code [e][*][b]}<br>
	 * <br> {@code b =} begin<br> {@code e =} end<br>
	 * <br>
	 * Horizontal, from [0,2] to [0,0]. Example: 引 = FbFBf<b>b</b>dB
	 */
	D,

	/**
	 * Unused.
	 */
	E,

	/**
	 * 
	 * <br>{@code [b][*][e]}<br>
	 * <br> {@code b =} begin<br> {@code e =} end<br>
	 * <br>
	 * Horizontal, from [0,0] to [0,2]. Example: ー = <b>F</b>
	 */
	F,

	/**
	 * 
	 * <br>{@code [e][ ][ ]}<br> {@code [ ][*][ ]}<br> {@code [ ][ ][b]}<br>
	 * <br> {@code b =} begin<br> {@code e =} end<br>
	 * <br>
	 * Diagonal, from [2,2] to [0,0]. Example: o = Aci<b>g</b>
	 */
	G,
	/**
	 * 
	 * <br>{@code [e]}<br> {@code [*]}<br> {@code [b]}<br>
	 * <br> {@code b =} begin<br> {@code e =} end<br>
	 * <br>
	 * Vertical, from [2,0] to [0,0]. Example: 0 = Cbag<b>h</b>i
	 */
	H,

	/**
	 * 
	 * <br>{@code [ ][ ][e]}<br> {@code [ ][*][ ]}<br> {@code [b][ ][ ]}<br>
	 * <br> {@code b =} begin<br> {@code e =} end<br>
	 * <br>
	 * Diagonal, from [2,0] to [0,2]. Example: レ = B<b>i</b>
	 */
	I,
}
