package net.ajaest.jdk.gui.auxi;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import net.ajaest.jdk.core.main.Messages;
import net.ajaest.jdk.core.winHandlers.MainWH;
import net.ajaest.jdk.data.kanji.KanjiTag;
import net.ajaest.jdk.data.kanji.ReadingEntry;
import net.ajaest.lib.swing.util.EditContextMenuEngine;

public class JPanelFactory {

	public static JScrollPane createToggleKanjiGrid(MainWH mwh, List<Integer> kl) {

		ExportJPanel jp = new ExportJPanel(kl, mwh);
		jp.setLayout(new GridLayout(0, 4));

		int v = ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
		int h = ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED;
		JScrollPane jsp = new JScrollPane(v, h);
		jsp.setViewportView(jp);
		jsp.getVerticalScrollBar().setUnitIncrement(16);

		return jsp;

	}

	public static JScrollPane createKanjiGrid(MainWH mwh, List<Integer> lk) {

		JPanel panel = new JPanel();

		panel.setLayout(new GridLayout(0, 4));

		int listPosition = 0;
		for (Integer k : lk) {

			KanjiButton b = new KanjiButton(mwh, k, listPosition);
			listPosition++;
			b.enableInvokeKanjiInformationWindowActionListener(true);
			panel.add(b);
		}
		int v = ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
		int h = ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED;
		JScrollPane jsp = new JScrollPane(v, h);
		jsp.setViewportView(panel);
		jsp.getVerticalScrollBar().setUnitIncrement(16);

		return jsp;
	}

	public static JScrollPane createKanjiReadingsList(Font font, KanjiTag kt, boolean extended, boolean romanized, EditContextMenuEngine eCtxMenE) {

		if (romanized) {
			kt = kt.toRomaji();
			font = new JPanel().getFont();
		}

		JPanel panel = new JPanel();

		panel.setFont(font);
		panel.setLayout(new GridLayout(0, 1));

		Map<String, List<String>> readingsMap = new HashMap<String, List<String>>();
		for (ReadingEntry re : kt.getReadings()) {
			readingsMap.put(re.getKey(), re.getElements());
		}

		JPanel onYomiPanel = new JPanel();
		onYomiPanel.setLayout(new BorderLayout());
		onYomiPanel.setBackground(Color.GREEN);
		JLabel jl = new JLabel(Messages.get(Messages.READ_JA_ON));
		jl.setVerticalAlignment(JLabel.CENTER);
		jl.setHorizontalAlignment(JLabel.CENTER);
		onYomiPanel.add(jl, BorderLayout.CENTER);
		panel.add(onYomiPanel);

		List<String> readings = readingsMap.get("ja_on");
		if (readings == null) {
			readings = new ArrayList<String>();
			readings.add("---");
		}

		addLabelsRowly(panel, readings, font, eCtxMenE);

		JPanel kunYomiPanel = new JPanel();
		kunYomiPanel.setLayout(new BorderLayout());
		kunYomiPanel.setBackground(Color.ORANGE);
		jl = new JLabel(Messages.get(Messages.READ_JA_KUN));
		jl.setVerticalAlignment(JLabel.CENTER);
		jl.setHorizontalAlignment(JLabel.CENTER);
		kunYomiPanel.add(jl, BorderLayout.CENTER);
		panel.add(kunYomiPanel);

		readings = readingsMap.get("ja_kun");
		if (readings == null) {
			readings = new ArrayList<String>();
			readings.add("---");
		}

		addLabelsRowly(panel, readings, font, eCtxMenE);

		if (extended) {

			JPanel nanori = new JPanel();
			nanori.setLayout(new BorderLayout());
			nanori.setBackground(Color.PINK);
			jl = new JLabel(Messages.get(Messages.READ_NANORI));
			jl.setVerticalAlignment(JLabel.CENTER);
			jl.setHorizontalAlignment(JLabel.CENTER);
			nanori.add(jl, BorderLayout.CENTER);
			panel.add(nanori);

			readings = readingsMap.get("nanori");
			if (readings == null) {
				readings = new ArrayList<String>();
				readings.add("---");
			}

			addLabelsRowly(panel, readings, font, eCtxMenE);

			// for (String s : readings) {
			// tempJL = new JLabel(s);
			// tempJL.setFont(font);
			// // tempJL.setVerticalAlignment(JLabel.CENTER);
			// tempJL.setHorizontalAlignment(JLabel.CENTER);
			// tempJP = new JPanel();
			// tempJP.add(tempJL);
			// panel.add(tempJP);
			// }

			// JPanel pinyin = new JPanel();
			// pinyin.setLayout(new BorderLayout());
			// pinyin.setBackground(Color.MAGENTA);
			// jl = new JLabel(Messages.get(Messages.READ_PINYIN));
			// jl.setVerticalAlignment(JLabel.CENTER);
			// jl.setHorizontalAlignment(JLabel.CENTER);
			// pinyin.add(jl, BorderLayout.CENTER);
			// panel.add(pinyin);
			//
			// readings = readingsMap.get("pinyin");
			// if (readings == null) {
			// readings = new ArrayList<String>();
			// readings.add("---");
			// }
			//
			// addLabelsRowly(panel, readings, font, eCtxMenE);
			//
			// JPanel korean = new JPanel();
			// korean.setLayout(new BorderLayout());
			// korean.setBackground(Color.CYAN);
			// jl = new JLabel(Messages.get(Messages.READ_KOREAN_H));
			// jl.setVerticalAlignment(JLabel.CENTER);
			// jl.setHorizontalAlignment(JLabel.CENTER);
			// korean.add(jl, BorderLayout.CENTER);
			// panel.add(korean);
			//
			// if (romanized)
			// readings = readingsMap.get("korean_r");
			// else
			// readings = readingsMap.get("korean_h");
			//
			// if (readings == null) {
			// readings = new ArrayList<String>();
			// readings.add("---");
			// }
			//
			// addLabelsRowly(panel, readings, font, eCtxMenE);
		}

		int v = ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
		int h = ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER;
		JScrollPane jsp = new JScrollPane(v, h);
		jsp.setViewportView(panel);
		jsp.getVerticalScrollBar().setUnitIncrement(16);

		return jsp;
	}
	private static List<JLabel> addLabelsRowly(JPanel panel, List<String> lines, Font font, EditContextMenuEngine eCtxMenE) {

		List<JLabel> labels = new ArrayList<JLabel>();

		JLabel tempJL;
		JPanel tempJP;

		for (String s : lines) {

			tempJL = new JLabel(s);
			tempJL.setFont(font);
			// tempJL.setVerticalAlignment(JLabel.CENTER);
			tempJL.setHorizontalAlignment(JLabel.CENTER);

			if (eCtxMenE != null)
				eCtxMenE.addContextMenu(tempJL);

			labels.add(tempJL);

			tempJP = new JPanel();
			tempJP.add(tempJL);
			panel.add(tempJP);
		}


		return labels;
	}

}