//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.jdk.tools.test.data.auxi;

import java.io.File;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;
import net.ajaest.jdk.data.auxi.AllowedStrokeLineEnum;
import net.ajaest.jdk.data.auxi.KanjiStroke;
import net.ajaest.jdk.data.kanji.KanjiGraph;
import net.ajaest.jdk.tools.parsers.StrokeOrderParser;
import net.ajaest.lib.data.OrderedSequenceTree;

import org.neodatis.odb.ODB;
import org.neodatis.odb.ODBFactory;
import org.neodatis.odb.Objects;
import org.neodatis.odb.core.query.criteria.Where;
import org.neodatis.odb.impl.core.query.criteria.CriteriaQuery;

//TODO: JavaDoc
public class testOrderedSequenceTree extends TestCase {

	OrderedSequenceTree<AllowedStrokeLineEnum, Integer> ost = null;
	OrderedSequenceTree<AllowedStrokeLineEnum, Integer> temp = null;

	// ***************************************
	// ///////////////////////////////////////
	// ///////// JUnit override methods
	// ///////////////////////////////////////
	// ***************************************

	protected void setUp() throws Exception {
		super.setUp();

		// void tree
		temp = new OrderedSequenceTree<AllowedStrokeLineEnum, Integer>(false);

		// full tree
		ost = new OrderedSequenceTree<AllowedStrokeLineEnum, Integer>(false);
		
		Map<Integer, KanjiGraph> kgmap = new StrokeOrderParser("resources/data/strokedata.txt", false).parse();

		List<AllowedStrokeLineEnum> sequence= null;
		for (Integer i : kgmap.keySet()) {
			sequence = getLineSequence(kgmap.get(i));
			
			ost.add(sequence, i);
		}

		File testDatabase = new File("resources/test/testSequenceTree.odb");
		if (testDatabase.exists())
			testDatabase.delete();

		ODB odb = ODBFactory.open("resources/test/testSequenceTree.odb");
		odb.store(ost);

		CriteriaQuery mainStrokeTree = new CriteriaQuery(OrderedSequenceTree.class, Where.isNull("parent"));
		Objects<OrderedSequenceTree<AllowedStrokeLineEnum, Integer>> graphTreeResult = odb.getObjects(mainStrokeTree);

		System.out.println(graphTreeResult.size());
		odb.close();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	// ***************************************
	// ///////////////////////////////////////
	// ///////// Support methods
	// ///////////////////////////////////////
	// ***************************************

	public static List<AllowedStrokeLineEnum> getLineSequence(KanjiGraph kg) {

		List<AllowedStrokeLineEnum> sequence = new ArrayList<AllowedStrokeLineEnum>();

		for (KanjiStroke ks : kg.getStrokes()) {
			for (AllowedStrokeLineEnum asl : ks.getStrokeList()) {

				sequence.add(asl);
			}
		}

		return sequence;
	}

	public void printSequenceTree(OrderedSequenceTree<AllowedStrokeLineEnum, Integer> st) {

		StringBuilder b = new StringBuilder("ordered by ");
		b.append(st.getSequenceValue());
		b.append(": [");

		char c;
		for (Integer obj : st.getFinalNodes()) {
			c = (char) obj.intValue();
			b.append(c);
			b.append(", ");
		}
		if (b.lastIndexOf(",") == -1)
			b.append(']');
		else
			b.setCharAt(b.length() - 1, ']');

		System.out.println(b.toString());
	}
	// ***************************************
	// ///////////////////////////////////////
	// ///////// Test constructors
	// ///////////////////////////////////////
	// ***************************************

	public void testConstructor() {

		boolean azzert = true;

		temp = new OrderedSequenceTree<AllowedStrokeLineEnum, Integer>(true);

		azzert = azzert && temp.getFinalNodes().getClass().equals(LinkedList.class);

		temp = new OrderedSequenceTree<AllowedStrokeLineEnum, Integer>(false);

		azzert = azzert && temp.getFinalNodes().getClass().equals(ArrayList.class);

		assertTrue(azzert);

	}

	// ***************************************
	// ///////////////////////////////////////
	// ///////// Test methods
	// ///////////////////////////////////////
	// ***************************************



	public void testGetSubTrees() {

		temp.addSubTree(AllowedStrokeLineEnum.E, ost);
		
		assertEquals(temp.getSubTrees().get(AllowedStrokeLineEnum.E), ost);
	}


	public void testGetParent() {

		temp.addSubTree(AllowedStrokeLineEnum.E, ost);

		assertEquals(temp, ost.getParent());
	}

	public void testAdd() {

		OrderedSequenceTree<AllowedStrokeLineEnum, Integer> test = temp.getSubTree(AllowedStrokeLineEnum.B);

		boolean azzert = (test == null);

		List<AllowedStrokeLineEnum> asl = new ArrayList<AllowedStrokeLineEnum>();
		asl.add(AllowedStrokeLineEnum.B);
		asl.add(AllowedStrokeLineEnum.B);
		asl.add(AllowedStrokeLineEnum.F);
		asl.add(AllowedStrokeLineEnum.A);
		asl.add(AllowedStrokeLineEnum.A);
		asl.add(AllowedStrokeLineEnum.C);
		asl.add(AllowedStrokeLineEnum.B);
		asl.add(AllowedStrokeLineEnum.A);
		asl.add(AllowedStrokeLineEnum.F);
		asl.add(AllowedStrokeLineEnum.F);
		asl.add(AllowedStrokeLineEnum.B);
		asl.add(AllowedStrokeLineEnum.A);
		asl.add(AllowedStrokeLineEnum.C);

		temp.add(asl, (int) '案');

		test = temp.getSubTree(AllowedStrokeLineEnum.B);
		test = test.getSubTree(AllowedStrokeLineEnum.B);
		test = test.getSubTree(AllowedStrokeLineEnum.F);
		test = test.getSubTree(AllowedStrokeLineEnum.A);
		test = test.getSubTree(AllowedStrokeLineEnum.A);
		test = test.getSubTree(AllowedStrokeLineEnum.C);
		test = test.getSubTree(AllowedStrokeLineEnum.B);
		test = test.getSubTree(AllowedStrokeLineEnum.A);
		test = test.getSubTree(AllowedStrokeLineEnum.F);
		test = test.getSubTree(AllowedStrokeLineEnum.F);
		test = test.getSubTree(AllowedStrokeLineEnum.B);
		test = test.getSubTree(AllowedStrokeLineEnum.A);
		test = test.getSubTree(AllowedStrokeLineEnum.C);

		azzert = azzert && test.getFinalNodes().size() != 0;
		azzert = azzert && test.getFinalNodes().get(0).equals((int) '案');

		test = test.getParent();
		test = test.getParent();
		test = test.getParent();
		test = test.getParent();
		test = test.getParent();
		test = test.getParent();
		test = test.getParent();
		test = test.getParent();
		test = test.getParent();
		test = test.getParent();
		test = test.getParent();
		test = test.getParent();
		test = test.getParent();
		// test = test.getSubTree(AllowedStrokeLineEnum.C);

		azzert = azzert && test.equals(temp) && test.getParent() == null;

		assertTrue(azzert);

	}

	public void testSetFinalNode() {


		boolean azzert = false;

		try {
			temp.setFinalNode(2, 32);
		} catch (IndexOutOfBoundsException e) {
			azzert = true;
		}
		
		temp.addFinalNode(1);
		temp.addFinalNode(2);
		temp.addFinalNode(666);
		
		temp.setFinalNode(2, 32);
		
		azzert = azzert && temp.getFinalNodes().get(2) != 666;
		azzert = azzert && temp.getFinalNodes().get(2) == 32;

		assertTrue(azzert);
	}
	public void testAddFinalNode() {

		int previousSize = temp.getFinalNodes().size();

		temp.addFinalNode(32);

		boolean azzert = temp.getFinalNodes().size() == previousSize + 1;
		azzert = azzert && temp.getFinalNodes().get(temp.getFinalNodes().size() - 1) == 32;

		assertTrue(azzert);
		
	}

	public void testAddSubTree() {

		boolean azzert = temp.getSubTree(AllowedStrokeLineEnum.A) == null;
		temp.addSubTree(AllowedStrokeLineEnum.A, ost);
		azzert = azzert && temp.getSubTree(AllowedStrokeLineEnum.A).equals(ost);

		assertTrue(azzert);
	}

	public void testRemoveSubTree() {

		temp.addSubTree(AllowedStrokeLineEnum.A, ost);
		boolean azzert = temp.getSubTree(AllowedStrokeLineEnum.A).equals(ost);

		temp.removeSubTree(AllowedStrokeLineEnum.A);

		azzert = azzert && temp.getSubTree(AllowedStrokeLineEnum.A) == null;

		assertTrue(azzert);
	}

	public void testRemoveFinalNodeInt() {

		temp.addFinalNode(1);
		temp.addFinalNode(2);
		temp.addFinalNode(666);

		int previousSize = temp.getFinalNodes().size();

		boolean azzert = temp.getFinalNodes().get(1) == 2;

		temp.removeFinalNode(1);

		azzert = azzert && temp.getFinalNodes().size() == previousSize - 1;

		azzert = azzert && temp.getFinalNodes().get(0) == 1;
		azzert = azzert && temp.getFinalNodes().get(1) == 666;

		assertTrue(azzert);

	}

	public void testRemoveFinalNodeE() {

		temp.addFinalNode(1);
		temp.addFinalNode(2);
		temp.addFinalNode(666);

		int previousSize = temp.getFinalNodes().size();

		boolean azzert = !temp.removeFinalNode(new Integer(4));

		azzert = azzert && temp.removeFinalNode(new Integer(2));

		azzert = azzert && temp.getFinalNodes().size() == previousSize - 1;

		azzert = azzert && temp.getFinalNodes().get(0) == 1;
		azzert = azzert && temp.getFinalNodes().get(1) == 666;

		assertTrue(azzert);
	}

	public void testSearch() {

		// complicated existing sequence
		List<AllowedStrokeLineEnum> 案 = new ArrayList<AllowedStrokeLineEnum>();

		案.add(AllowedStrokeLineEnum.B);
		案.add(AllowedStrokeLineEnum.B);
		案.add(AllowedStrokeLineEnum.F);
		案.add(AllowedStrokeLineEnum.A);
		案.add(AllowedStrokeLineEnum.A);
		案.add(AllowedStrokeLineEnum.C);
		案.add(AllowedStrokeLineEnum.B);
		案.add(AllowedStrokeLineEnum.A);
		案.add(AllowedStrokeLineEnum.F);
		案.add(AllowedStrokeLineEnum.F);
		案.add(AllowedStrokeLineEnum.B);
		案.add(AllowedStrokeLineEnum.A);
		案.add(AllowedStrokeLineEnum.C);

		List<Integer> results = ost.search(案);

		boolean azzert = results.contains((int) '案');

		// first level
		List<AllowedStrokeLineEnum> 一 = new ArrayList<AllowedStrokeLineEnum>();
		// ichi kanji
		一.add(AllowedStrokeLineEnum.F);

		results = ost.search(一);

		azzert = azzert && results.contains((int) '一');

		// void search, returns the current sequencetree final nodes

		List<AllowedStrokeLineEnum> voiz = new ArrayList<AllowedStrokeLineEnum>();

		ost.addFinalNode(666);
		results = ost.search(voiz);

		azzert = azzert && results.size() == 1;
		azzert = azzert && results.get(0) == 666;

		assertTrue(azzert);

		// no exists, return void

		List<AllowedStrokeLineEnum> noexists = new ArrayList<AllowedStrokeLineEnum>();
		noexists.add(AllowedStrokeLineEnum.F);
		noexists.add(AllowedStrokeLineEnum.F);
		noexists.add(AllowedStrokeLineEnum.F);
		noexists.add(AllowedStrokeLineEnum.F);

		results = ost.search(noexists);

		azzert = azzert && results.size() == 0;

		assertTrue(azzert);
	}

	public static void main(String... args) throws Exception {

		testOrderedSequenceTree t1 = new testOrderedSequenceTree();
		OrderedSequenceTree<AllowedStrokeLineEnum, Integer> test;
		t1.setUp();
		test = t1.ost;

		ODB odb = ODBFactory.open("resources/test/aaa.odb");
		odb.store(test);

	}

}
