//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.lib.data;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.NavigableMap;
import java.util.NavigableSet;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;


//TODO: JavaDoc
public class CollectionsExtra {
	// Suppresses default constructor, ensuring non-instantiability.
	private CollectionsExtra() {};

	public static <E> NavigableSet<E> quickHashCodeNavigableSet(NavigableSet<E> ns) {

		return new BufferedHashCodeNavigableSet<E>(ns);
	}

	public static <E> Set<E> quickHashCodeSet(Set<E> s) {

		return new BufferedHashCodeSet<E>(s);
	}
	
	public static <C1 extends Collection<C2>, C2 extends Collection<E>, E> List<E> plainCollection(C1 col) {

		List<E> plained = new ArrayList<E>();

		for (C2 col1 : col) {
			for (E elem : col1) {
				plained.add(elem);
			}
		}

		return plained;
	}

	public static <MAP extends NavigableMap<K1, MAP1>, MAP1 extends Map<?, C1>, C1 extends Collection<E>, K1, E> TreeMap<K1, NavigableSet<E>> plainNavigableMap(MAP map) {

		TreeMap<K1, NavigableSet<E>> plained = new TreeMap<K1, NavigableSet<E>>();

		NavigableSet<E> tempCol;
		for (K1 key : map.keySet()) {
			for (C1 ret : map.get(key).values()) {
				tempCol = plained.get(key);
				if (tempCol == null) {
					tempCol = new TreeSet<E>();
					plained.put(key, tempCol);
				}
				tempCol.addAll(ret);
			}
		}

		return plained;
	}

	
	private static class BufferedHashCodeNavigableSet<E> extends BufferedSortedSet<E> implements NavigableSet<E>{

		private NavigableSet<E> original;
		private BufferedHashCodeNavigableSet<E> thisRef;

		public BufferedHashCodeNavigableSet(NavigableSet<E> original) {
			super(original);

			this.original = original;
			thisRef = this;
		}

		@Override
		public E ceiling(E e) {
			return original.ceiling(e);
		}

		@Override
		public Iterator<E> descendingIterator() {

			return new QuickHashIterator<E>(this, original.descendingIterator());
		}

		@Override
		public NavigableSet<E> descendingSet() {

			BufferedHashCodeNavigableSet<E> ds = new BufferedHashCodeNavigableSet<E>(original.descendingSet());
			ds.addModifiedPolicy(new Runnable() {
				@Override
				public void run() {
					thisRef.triggerModified();
				}
			});
			
			return ds;
		}

		@Override
		public E floor(E e) {

			return original.floor(e);
		}

		@Override
		public NavigableSet<E> headSet(E toElement, boolean inclusive) {

			BufferedHashCodeNavigableSet<E> ds = new BufferedHashCodeNavigableSet<E>(original.headSet(toElement, inclusive));
			ds.addModifiedPolicy(new Runnable() {
				@Override
				public void run() {
					thisRef.triggerModified();
				}
			});
			
			return ds;
		}

		@Override
		public E higher(E e) {

			return original.higher(e);
		}

		@Override
		public E lower(E e) {

			return original.lower(e);
		}

		@Override
		public E pollFirst() {

			triggerModified();

			return original.pollFirst();
		}

		@Override
		public E pollLast() {

			triggerModified();

			return original.pollLast();
		}

		@Override
		public NavigableSet<E> subSet(E fromElement, boolean fromInclusive,
				E toElement, boolean toInclusive) {
			
			BufferedHashCodeNavigableSet<E> ds = new BufferedHashCodeNavigableSet<E>(original.subSet(fromElement, fromInclusive, toElement, toInclusive));
			ds.addModifiedPolicy(new Runnable() {
				@Override
				public void run() {
					thisRef.triggerModified();
				}
			});
			

			return ds;
		}

		@Override
		public NavigableSet<E> tailSet(E fromElement, boolean inclusive) {

			BufferedHashCodeNavigableSet<E> ds = new BufferedHashCodeNavigableSet<E>(
					original.tailSet(fromElement, inclusive));
			ds.addModifiedPolicy(new Runnable() {
				@Override
				public void run() {
					thisRef.triggerModified();
				}
			});

			return ds;
		}
		
	}
	
	private static class BufferedSortedSet<E> extends BufferedHashCodeSet<E> implements SortedSet<E>  {

		private SortedSet<E> original;
		private BufferedSortedSet<E> thisRef;

		public BufferedSortedSet(SortedSet<E> original) {
			super(original);

			this.original = original;
			thisRef = this;
		}

		@Override
		public Comparator<? super E> comparator() {

			return original.comparator();
		}

		@Override
		public E first() {

			return original.first();
		}

		@Override
		public SortedSet<E> headSet(E toElement) {
			
			BufferedSortedSet<E> bss = new BufferedSortedSet<E>(original.headSet(toElement));
			
			bss.addModifiedPolicy(new Runnable() {
				@Override
				public void run() {
					thisRef.triggerModified();
				}
			});
			
			return bss;
		}

		@Override
		public E last() {

			return original.last();
		}

		@Override
		public SortedSet<E> subSet(E fromElement, E toElement) {

			BufferedSortedSet<E> bss = new BufferedSortedSet<E>(original.subSet(fromElement, toElement));
			
			bss.addModifiedPolicy(new Runnable() {
				@Override
				public void run() {
					thisRef.triggerModified();
				}
			});
			
			return bss;
		}

		@Override
		public SortedSet<E> tailSet(E fromElement) {
			
			BufferedSortedSet<E> bss = new BufferedSortedSet<E>(original.tailSet(fromElement));
			
			bss.addModifiedPolicy(new Runnable() {
				@Override
				public void run() {
					thisRef.triggerModified();
				}
			});
			
			return bss;
		}

	}

	private static class BufferedHashCodeSet<E> implements Set<E> {

		protected Set<E> original;
		protected boolean modified;
		protected Set<Runnable> modifyPolicies;
		protected Set<Runnable> hashCodeRanPolicies;
		protected int hashCode;

		public BufferedHashCodeSet(Set<E> set) {

			this.original = set;
			modified = true;

			modifyPolicies = new HashSet<Runnable>();
			modifyPolicies.add(new Runnable() {
				@Override
				public void run() {
					modified = true;
				}
			});

			hashCodeRanPolicies = new HashSet<Runnable>();
			hashCodeRanPolicies.add(new Runnable() {

				@Override
				public void run() {
					modified = false;

				}
			});
		}

		@Override
		public boolean add(E e) {

			triggerModified();

			return original.add(e);
		}

		@Override
		public boolean addAll(Collection<? extends E> c) {

			triggerModified();

			return original.addAll(c);
		}

		@Override
		public void clear() {
			triggerModified();

			original.clear();
		}

		@Override
		public boolean contains(Object o) {

			return original.contains(o);
		}

		@Override
		public boolean containsAll(Collection<?> c) {

			return original.containsAll(c);
		}

		@Override
		public boolean isEmpty() {

			return original.isEmpty();
		}

		@Override
		public Iterator<E> iterator() {

			return new QuickHashIterator<E>(this);
		}

		@Override
		public boolean remove(Object o) {

			triggerModified();

			return original.remove(o);
		}

		@Override
		public boolean removeAll(Collection<?> c) {

			triggerModified();

			return original.removeAll(c);
		}

		@Override
		public boolean retainAll(Collection<?> c) {

			triggerModified();


			return original.retainAll(c);
		}

		@Override
		public int size() {

			return original.size();
		}

		@Override
		public Object[] toArray() {

			return original.toArray();
		}

		@Override
		public <T> T[] toArray(T[] a) {

			return original.toArray(a);
		}

		public void addModifiedPolicy(Runnable r) {
			modifyPolicies.add(r);
		}

		@SuppressWarnings("unused")
		public void addHashCodeRanPolicy(Runnable r) {
			hashCodeRanPolicies.add(r);
		}

		@SuppressWarnings("unused")
		public void removeModifiedPolicy(Runnable r) {
			modifyPolicies.remove(r);
		}

		@SuppressWarnings("unused")
		public void removeHashCodeRanPolicy(Runnable r) {
			hashCodeRanPolicies.remove(r);
		}

		public void triggerModified() {
			for (Runnable r : modifyPolicies)
				r.run();
		}

		public void triggerHashCodeRun() {
			for (Runnable r : hashCodeRanPolicies)
				r.run();
		}

		@Override
		public int hashCode() {
			
			if(modified){
				hashCode = original.hashCode();
				triggerHashCodeRun();
			}

			return hashCode;
		}

		@Override
		public boolean equals(Object obj) {

			return original.equals(obj);
		}
	}

	private static class QuickHashIterator<E> implements Iterator<E> {

		Iterator<E> original;
		BufferedHashCodeSet<E> originalSet;

		public QuickHashIterator(BufferedHashCodeSet<E> originalSet) {

			this.originalSet = originalSet;
			original = originalSet.original.iterator();
		}

		public QuickHashIterator(BufferedHashCodeSet<E> originalSet,
				Iterator<E> it) {

			this.originalSet = originalSet;
			original = it;
		}

		@Override
		public boolean hasNext() {

			return original.hasNext();
		}

		@Override
		public E next() {

			return original.next();
		}

		@Override
		public void remove() {

			original.remove();
			originalSet.triggerModified();
		}

	}
}
