//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.lib.data;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Set;
import java.util.TreeSet;

//TODO: JavaDoc
public class ComparableTreeSet<E> extends TreeSet<E> implements Comparable<ComparableTreeSet<E>> {

	private static final long serialVersionUID = 5488350458130734346L;

	private Comparator<E> defComparator;
	/**
	 * determines if the {@code <E>} parameter is comparable
	 */
	private Boolean isComparable = null;

	public ComparableTreeSet() {
		this((Comparator<E>) null);
	}

	public ComparableTreeSet(Collection<? extends E> c) {
		this(null, c);
	}
	
	public ComparableTreeSet(Comparator<E> defComparator) {
		this(defComparator, new ArrayList<E>());
	}
	@SuppressWarnings("unchecked")
	public ComparableTreeSet(Comparator<E> defComparator, Collection<? extends E> c) {
		super(c);

		if (defComparator == null) {

			this.defComparator = new Comparator<E>() {
				@Override
				public int compare(E o1, E o2) {

					int compare;

					if (isComparable == null)
						isComparable = isComparable(o1);

					if (isComparable) {

						Comparable<E> c1 = (Comparable<E>) o1;
						compare = c1.compareTo(o2);

					} else
						compare = 0;

					return compare;
				}
			};
		} else {
			this.defComparator = defComparator;
		}
			
	}

	@Override
	public int compareTo(ComparableTreeSet<E> o) {

		int returns;

		if (this.size() != 0 && o.size() != 0) {
			Set<Pair<E, ElementOwner>> both = new HashSet<Pair<E, ElementOwner>>();
			Set<Pair<E, ElementOwner>> set1 = new HashSet<Pair<E, ElementOwner>>();
			Set<Pair<E, ElementOwner>> set2 = new HashSet<Pair<E, ElementOwner>>();

			Set<E> tempBoth = new HashSet<E>(this);
			tempBoth.retainAll(o);
			for (E obj : tempBoth)
				both.add(new Pair<E, ElementOwner>(obj, ElementOwner.Both));
			{
				Set<E> tempSet1 = new HashSet<E>(this);
				tempSet1.removeAll(tempBoth);
				for (E obj : tempSet1)
					set1.add(new Pair<E, ElementOwner>(obj, ElementOwner.Set1));
			}

			{
				Set<E> tempSet2 = new HashSet<E>(o);
				tempSet2.removeAll(tempBoth);
				for (E obj : tempSet2)
					set2.add(new Pair<E, ElementOwner>(obj, ElementOwner.Set2));
			}
			Set<Pair<E, ElementOwner>> sortedSet = new TreeSet<Pair<E, ElementOwner>>(new EnumPairComparator());

			sortedSet.addAll(both);
			sortedSet.addAll(set1);
			sortedSet.addAll(set2);

			Double set1Punct = 0d;
			Double set2Punct = 0d;

			if (sortedSet.size() > 0) {

				int set1elementsAfter = this.size();
				int set2elementsAfter = o.size();

				for (Pair<E, ElementOwner> pa : sortedSet) {

					switch (pa.getSecond()) {
						case Set1 :
							set1Punct += set2elementsAfter;
							set1elementsAfter--;
							break;
						case Set2 :
							set2Punct += set1elementsAfter;
							set2elementsAfter--;
							break;
						case Both :
							set1elementsAfter--;
							set2elementsAfter--;
							set1Punct += set2elementsAfter;
							set2Punct += set1elementsAfter;
							break;
					}
				}

			}

			int compare = set1Punct.compareTo(set2Punct);

			double orderCoeficient = compare >= 1 ? set2Punct / set1Punct : set1Punct / set2Punct;
			returns = ((int) Math.signum(compare)) * (int) (100 * (1 - orderCoeficient));
		} else {
			if(this.size()==0 && o.size()!=0)
				returns = -100;
			else if (this.size() != 0 && o.size() == 0)
				returns = +100;
			else
				returns = 0;
		}

		return returns;
	}
	
	private enum ElementOwner {
		Set1, Set2, Both
	}
	
	@Override
	public Comparator<E> comparator() {
		return defComparator;
	}

	private class EnumPairComparator implements Comparator<Pair<E, ElementOwner>> {

		@Override
		public int compare(Pair<E, ElementOwner> o1, Pair<E, ElementOwner> o2) {

			return defComparator.compare(o1.getFirst(), o2.getFirst());
		}
	}

	private boolean isComparable(Object o) {
		
		boolean isComparable = true;
		
		try {
			o.getClass().getMethod("compareTo", o.getClass());
		} catch (SecurityException e) {
			isComparable = false;
		} catch (NoSuchMethodException e) {
			isComparable = false;
		}

		return isComparable;
	}

	public static void main(String... args) {

		ComparableTreeSet<Integer> s1 = new ComparableTreeSet<Integer>();
		s1.add(1);

		ComparableTreeSet<Integer> s2 = new ComparableTreeSet<Integer>();
		s2.add(2);

		System.out.println(s1.compareTo(s2));

	}
}
