//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.lib.data;

/**
 * 
 * Generic class that stores two elements.
 * 
 * @author Luis Alfonso Arce González
 * 
 * @param <E1>
 *            First element
 * @param <E2>
 *            Second element
 */
public class Pair<E1, E2> {

	private E1 first;
	private E2 second;

	public Pair(E1 first, E2 second) {
		super();
		this.first = first;
		this.second = second;
	}

	/**
	 * Returns the first element.Read the constructor or class info for further
	 * information about the meaning of "first".
	 */
	public E1 getFirst() {
		return first;
	}

	/**
	 * Stores the first element overriding the previous one.Read the constructor
	 * or class info for further information about the meaning of "first".
	 * 
	 * @param first
	 */
	public void setFirst(E1 first) {
		this.first = first;
	}

	/**
	 * Returns the second element.Read the constructor or class info for further
	 * information about the meaning of "second".
	 */
	public E2 getSecond() {
		return second;
	}

	/**
	 * Stores the second element replacing any element stored before.Read the
	 * constructor or class info for further information about the meaning of
	 * "second".
	 * 
	 * @param second
	 */
	public void setSecond(E2 second) {
		this.second = second;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((first == null) ? 0 : first.hashCode());
		result = prime * result + ((second == null) ? 0 : second.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Pair<?, ?> other = (Pair<?, ?>) obj;
		if (first == null) {
			if (other.first != null)
				return false;
		} else if (!first.equals(other.first))
			return false;
		if (second == null) {
			if (other.second != null)
				return false;
		} else if (!second.equals(other.second))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "[first=" + first + ", second=" + second + "]";
	}

}
