//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.lib.swing.util.action;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;

/**
 * Action class that triggers actions from multiple {@code Component} instances
 * when it's triggered, concatenating the result stored in the clipboard for
 * every instance and sending it as text to a the specified action.
 * 
 * @author Luis Alfonso Arce González
 * 
 */
public abstract class MultiComponentAction extends AbstractAction {


	private static final long serialVersionUID = -224352238973810520L;
	
	private Action sendTo;

	private List<Action> actions;
	private List<Component> components;

	public MultiComponentAction(Action sendTo, List<Action> actions, List<Component> components) {

		if (actions.size() != components.size())
			throw new IndexOutOfBoundsException("size mismatch between action list and component list");

		this.sendTo = sendTo;

		this.actions = actions;
		this.components = components;
	}

	@Override
	public void actionPerformed(ActionEvent e) {

		for (int i = 0; i < actions.size(); i++) {
			e.setSource(components.get(i));
			actions.get(i).actionPerformed(e);
			appendAction(components.get(i));
		}

		sendTo.actionPerformed(prepareActionEventForFinalAction(e));
	}

	protected abstract void appendAction(Component component);
	protected abstract ActionEvent prepareActionEventForFinalAction(ActionEvent e);

}
