//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.lib.tools.test.data;

import java.util.HashSet;
import java.util.Iterator;
import java.util.NavigableSet;
import java.util.Set;
import java.util.TreeSet;

import junit.framework.TestCase;
import net.ajaest.lib.data.CollectionsExtra;

public class TestCollectionsExtra extends TestCase {

	NavigableSet<Integer> notModified;
	NavigableSet<Integer> modified;

	@Override
	protected void setUp() {
		
		notModified = new TreeSet<Integer>();
		
		int count = ramdomFrom0to9999();
		while (count < 1000)
			count = ramdomFrom0to9999();

		for (int i = 0; i < count; i++) {
			notModified.add(ramdomFrom0to9999()); // ramdom from 0 to 999
		}
		
		System.out.println(notModified.size());

		// for (Integer i : notModified) {
		// System.out.println(i);
		// }
		//		
		modified = CollectionsExtra.quickHashCodeNavigableSet(notModified);

	}

	public void test_contains() {

		for (Integer i : notModified) {
			assertTrue(modified.contains(i));
		}

		Set<Integer> notContained = new HashSet<Integer>();

		for (int i = 0; i < 100; i++) {
			notContained.add(ramdomNotContained(notModified));
		}

		for (Integer i : notContained) {
			assertFalse(modified.contains(i));
		}
	}

	public void test_add() {

		Integer notContained = ramdomNotContained(notModified);

		assertFalse(notModified.contains(notContained));
		assertFalse(modified.contains(notContained));

		modified.add(notContained);

		assertTrue(notModified.contains(notContained));
		assertTrue(modified.contains(notContained));

		assertEquals(notModified, modified);

		checkHashCode("add");
	}

	public void test_containsAll() {

		Set<Integer> notContained = new HashSet<Integer>();
		
		for (int i = 0; i < 5; i++) {
			notContained.add(ramdomNotContained(notModified));
		}

		assertTrue(modified.containsAll(notModified));
		
		assertFalse(modified.containsAll(notContained));

		notContained.add(notModified.first());

		assertFalse(modified.containsAll(notContained));
		
		checkHashCode("containsAll");

	}

	public void test_addAll() {

		Set<Integer> notContained = new HashSet<Integer>();

		for (int i = 0; i < 30; i++) {
			notContained.add(ramdomNotContained(notModified));
		}

		assertFalse(notModified.containsAll(notContained));
		assertFalse(modified.containsAll(notContained));

		modified.addAll(notContained);

		assertTrue(notModified.containsAll(notContained));
		assertTrue(modified.containsAll(notContained));

		assertEquals(notModified, modified);

		checkHashCode("addAll");

	}

	public void test_celing() {

		for (int i = 0; i < 1000; i++) {
			assertEquals(notModified.ceiling(i), modified.ceiling(i));
		}

		checkHashCode("ceiling");
	}

	public void test_clear() {

		assertEquals(modified, notModified);

		modified.clear();
		notModified.clear();

		assertEquals(modified, notModified);
		assertTrue(modified.size() == 0);
		assertTrue(notModified.size() == 0);

		checkHashCode("clear");
	}


	public void test_comparator() {
		assertEquals(modified.comparator(), notModified.comparator());

		checkHashCode("clear");
	}

	public void test_descendingIterator() {

		Iterator<Integer> it1, it2;
		it1 = notModified.descendingIterator();
		it2 = modified.descendingIterator();
		
		while (it1.hasNext() && it2.hasNext()) {
			assertEquals(it1.hasNext(), it2.hasNext());
			assertEquals(it1.next(), it2.next());
		}

		setUp();

		it1 = notModified.descendingIterator();
		it2 = modified.descendingIterator();

		while (it2.hasNext()) {
			it2.next();
			it2.remove();
		}
		
		assertTrue(notModified.size() == 0);
		assertTrue(modified.size()==0);

		checkHashCode("descendingIterator");
	}

	public void test_descendingSet() {

		Iterator<Integer> it1, it2;
		it1 = notModified.descendingSet().iterator();
		it2 = modified.descendingSet().iterator();

		while (it1.hasNext() && it2.hasNext()) {
			assertEquals(it1.hasNext(), it2.hasNext());
			assertEquals(it1.next(), it2.next());
		}

		setUp();

		it1 = notModified.descendingSet().iterator();
		it2 = modified.descendingSet().iterator();

		while (it2.hasNext()) {
			it2.next();
			it2.remove();
		}

		assertTrue(notModified.size() == 0);
		assertTrue(modified.size() == 0);

		checkHashCode("descendingSet");
	}

	public void test_equals() {
		
		Iterator<Integer> it1, it2;
		it1 = notModified.descendingSet().iterator();
		it2 = modified.descendingSet().iterator();

		while (it1.hasNext() && it2.hasNext()) {
			assertEquals(it1.hasNext(), it2.hasNext());
			assertEquals(it1.next(), it2.next());
		}
		
		assertTrue(modified.equals(notModified));

		checkHashCode("equals");
	}

	public void test_floor() {

		for (int i = 0; i < 1000; i++) {
			assertEquals(notModified.floor(i), modified.floor(i));
		}

		checkHashCode("floor");
	}

	/*public void test_hashCode() {

		assertEquals(modified.hashCode(), notModified.hashCode());
		System.out.println("Testing hashcode for 「hashcode」");
		
		NavigableSet<Long> big = new TreeSet<Long>();
		for (long i = 0; i < Integer.MAX_VALUE / 1000; i++)
			big.add(i);

		System.out.println("\tInitialized, size =" + big.size());

		big = CollectionsExtra.quickHashCodeNavigableSet(big);

		long b1, e1, b2, e2, t1, t2;
		b1 = System.nanoTime();
		big.hashCode();
		e1 = System.nanoTime();
		b2 = System.nanoTime();
		big.hashCode();
		e2 = System.nanoTime();

		t1 = e1 - b1;
		t2 = e2 - b2;

		System.out.println("\t1-Hashcode「" + "hashcode" + "」: 「" + t1
				+ "」 vs 「" + t2 + "」\t-> notBuffered, buffered");

	}*/

	public void test_headSet() {

		Integer first = ramdomFrom0to9999();
		while (notModified.floor(first) == null)
			first = ramdomFrom0to9999();

		Iterator<Integer> it1, it2;
		it1 = notModified.headSet(first).iterator();
		it2 = modified.headSet(first).iterator();

		while (it1.hasNext() && it2.hasNext()) {
			assertEquals(it1.hasNext(), it2.hasNext());
			assertEquals(it1.next(), it2.next());
		}

		setUp();

		while (notModified.floor(first) == null)
			first = ramdomFrom0to9999();

		it1 = notModified.headSet(first).iterator();
		it2 = modified.headSet(first).iterator();

		while (it2.hasNext()) {
			it2.next();
			it2.remove();
		}

		assertFalse(notModified.size() == 0);
		assertFalse(modified.size() == 0);

		checkHashCode("headSet");
	}

	// TODO: finish

	// ________________________________
	// ________________________________

	public void checkHashCode(String s) {
		
		System.out.println("Testing hashcode for 「" + s + "」");
		
		Long b1, e1, b2, e2,t1,t2;
		
		Integer i = modified.pollFirst();
		modified.add(i);

		b1 = System.nanoTime();
		modified.hashCode();
		e1 = System.nanoTime();

		b2 = System.nanoTime();
		modified.hashCode();
		e2 = System.nanoTime();

		t1 = e1-b1;
		t2 = e2-b2;
		
		System.out.println("\t1-Hashcode「" + s + "」: 「" + t1 + "」 vs 「" + t2
				+ "」\t-> notBuffered, buffered");
		
		b1 = System.nanoTime();
		modified.hashCode();
		e1 = System.nanoTime();

		b2 = System.nanoTime();
		modified.hashCode();
		e2 = System.nanoTime();

		t1 = e1-b1;
		t2 = e2-b2;
		
		System.out.println("\t2-Hashcode「" + s + "」: 「" + t1 + "」 vs 「" + t2
				+ "」 \t-> buffered, buffered");
		
		b1 = System.nanoTime();
		modified.hashCode();
		e1 = System.nanoTime();

		i = modified.pollFirst();
		modified.add(i);

		b2 = System.nanoTime();
		modified.hashCode();
		e2 = System.nanoTime();

		t1 = e1-b1;
		t2 = e2-b2;

		System.out.println("\t3-Hashcode「" + s + "」: 「" + t1 + "」 vs 「" + t2
				+ "」\t-> buffered, notBuffered\n");
	}

	public Integer ramdomFrom0to9999() {
		return (int)(Math.random() * Math.pow(10,(int)((Math.random()-0.001) * 5)));
	}

	public Integer ramdomNotContained(NavigableSet<Integer> i) {

		Integer notContained = ramdomFrom0to9999();

		while(i.contains(notContained))
			notContained = ramdomFrom0to9999();

		return notContained;
	}

	public static void main(String... args) {
		TestCollectionsExtra tce = new TestCollectionsExtra();
		tce.test_add();
	}

}
